/**
* user:  A1001
* fname: Aleksandar
* lname: Krastev
* task:  superstition
* score: 60.0
* date:  2017-11-24 11:38:30.593882
*/
#include <iostream>
#include <cstdint>
#include <vector>
#include <cassert>

const uint64_t MOD = 1e9 + 7;

template <class T>
struct Vector {
	std::vector<T> v;

	Vector() {}
	Vector(uint32_t elCnt, const T& val) {
		v.resize(elCnt);
		for (T &i : v) {
			i = val;
		}
	}
	Vector(const Vector<T> &a): v(a.v) {}

	Vector<T>& operator%= (uint64_t mod) {
		for (T &i : v) {
			i %= mod;
		}
		return *this;
	}

	Vector<T>& operator+= (const Vector<T> &u) {
		//std::cout << v.size() << " " << u.v.size() << "\n";
		assert(v.size() == u.v.size());
		for (uint32_t i = 0;i < v.size();i++) {
			v[i] += u.v[i];
			//v[i] %= MOD;
		}
		return *this;
	}

	friend Vector<T> operator+ (Vector<T> v, const Vector<T> &u) {
		return v += u;
	}
	

	friend T operator*(const Vector<T> &v1, const Vector<T> &v2) {
		T ans = T();
		assert(v1->size() == v2->size());
		for (uint32_t i = 0;i < v1.size();i++) {
			ans += v1->at(i) * v2->at(i);
			//ans %= MOD;
		}
		return ans;
	}
};

template <class T>
struct Matrix {
	std::vector<std::vector<T>> m;

	Matrix(){}
	Matrix(uint32_t h, uint32_t w, const T &val) {
		m.resize(h);
		for (uint32_t i = 0;i < h;i++) {
			m[i].resize(w);
			for (uint32_t j = 0;j < w;j++) {
				m[i][j] = val;
			}
		}
	}

	template<class U>
	Vector<U> operator*(const Vector<U> &v) {
		assert(m[0].size() == v.v.size());

		Vector<U> ans;
		ans.v.resize(m.size());

		for (uint32_t i = 0;i < m.size();i++) {
			//std::cout << m[i].size() << " " << v.v.size() << std::endl;
			ans.v[i] = m[i][0] * v.v[0];
			for (uint32_t j = 1;j < m[i].size();j++) {
				ans.v[i] += m[i][j] * v.v[j];
				ans.v[i] %= MOD;
			}
		}
		return ans;
	}
	Matrix<T>& operator %= (uint64_t num) {
		for (uint32_t i = 0;i < m.size();i++) {
			for (uint32_t j = 0;j < m[i].size();j++) {
				m[i][j] %= num;
			}
		}
		return *this;
	}
	Matrix<T>& operator += (const Matrix<T> &m1) {
		for (uint32_t i = 0;i < m.size();i++) {
			for (uint32_t j = 0;j < m[i].size();j++) {
				m[i][j] += m1.m[i][j];
			}
		}
		return *this;
	}

	friend Matrix<T> operator* (const Matrix<T> &m1, const Matrix<T> &m2) {
		Matrix<T> ans;
		ans.m.resize(m1.m.size());
		for (uint32_t i = 0;i < m1.m.size();i++) {
			ans.m[i].resize(m1.m[0].size());
			for (uint32_t j = 0;j < m2.m[0].size();j++) {
				ans.m[i][j] = m1.m[i][0] * m2.m[0][j];
				for (uint32_t k = 1;k < m1.m[0].size();k++) {
					ans.m[i][j] += m1.m[i][k] * m2.m[k][j];
					ans.m[i][j] %= MOD;
				}
			}
		}
		return ans;
	}
};

//Matrix<Vector<uint64_t>> m;

const uint32_t MAX_D = 1e4 + 10;
const uint32_t MAX_W = 10;
//const uint32_t MAX_W = 2;


Matrix<Matrix<uint64_t>> distMatrix;
uint32_t vCnt, eCnt;
uint32_t d, k;

const uint32_t MAX_N = 80;
std::vector<std::pair<uint32_t, uint32_t>> edges[MAX_N];


void read_matrix() {
	distMatrix = Matrix<Matrix<uint64_t>>(MAX_W, MAX_W, Matrix<uint64_t>(vCnt, vCnt, 0));
	for (uint32_t i = 1;i < MAX_W;i++) {
		for (uint32_t j = 0;j < vCnt;j++) {
			distMatrix.m[i][i - 1].m[j][j] = 1;
		}
	}

	for (uint32_t i = 0;i < eCnt;i++) {
		uint32_t a, b, w;
		std::cin >> a >> b >> w;
		a--;
		b--;
		w--;
		distMatrix.m[0][w].m[a][b]++;
		distMatrix.m[0][w].m[b][a]++;
	}
}

void read_edges() {
	for (uint32_t i = 0;i < eCnt;i++) {
		uint32_t a, b, w;
		std::cin >> a >> b >> w;
		a--;
		b--;
		edges[a].push_back(std::make_pair(b, w));
		edges[b].push_back(std::make_pair(a, w));
	}
}

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> vCnt >> eCnt >> d >> k;
	if (k < MAX_D) {
		read_edges();
	} else {
		read_matrix();
	}
}

uint64_t dp[MAX_D][MAX_N];

uint64_t solve_norm_dp() {
	for (uint32_t i = 0;i < vCnt;i++) {
		dp[0][i] = 1;
	}
	uint64_t ans = 0;
	for (uint32_t i = 1;i <= k;i++) {
		for (uint32_t j = 0;j < vCnt;j++) {
			for (auto e : edges[j]) {
				if (i >= e.second) {
					dp[i][j] += dp[i - e.second][e.first];
					if (dp[i][j] >= MOD) {
						dp[i][j] -= MOD;
					}
				}
			}
		}
		if (i % d == 0) {
			for (uint32_t j = 0;j < vCnt;j++) {
				ans += dp[i][j];
				ans %= MOD;
			}
		}
	}
	return ans;
}

template <class T>
T fast_pow(const T& a, uint32_t pow) {
	if (pow == 1) {
		return a;
	}
	T ans = fast_pow(a, pow / 2);
	ans = ans * ans;
	if (pow % 2) {
		ans = ans * a;
	}
	return ans;
}

uint64_t solve_mono_pow() {
	auto curMatrix = fast_pow(distMatrix, d);
	auto base = Vector<Vector<uint64_t>>(MAX_W, Vector<uint64_t>(vCnt, 0));
	base.v[0] = Vector<uint64_t>(vCnt, 1);
	auto ans = curMatrix * base;
	uint64_t ansNum = 0;
	for (uint32_t i = 0;i < vCnt;i++) {
		ansNum += ans.v[0].v[i];
		ansNum %= MOD;
	}
	return ansNum;
}

int main() {
	input();
	if (k < MAX_D) {
		std::cout << solve_norm_dp() << "\n";
	} else {
		std::cout << solve_mono_pow() << "\n";
	}
}
