/**
* user:  A1001
* fname: Aleksandar
* lname: Krastev
* task:  robots
* score: 0.0
* date:  2017-11-24 08:47:12.777755
*/
#include <iostream>
#include <cstdint>
#include <cstring>

const uint32_t MAX_L = 7;
const uint32_t MAX_N = 10;
const uint32_t MAX_K = 100;
const uint32_t INF = 1e9;
const uint32_t COMP_STATE_CNT = 2916;
const uint32_t MOD = (1U << (2 * MAX_L)) - 1;

uint32_t decompState[1U << (2 * MAX_L)];
uint32_t compState[1U << (2 * MAX_L)];

int32_t dX[] = {1, -1, 0, 0};
int32_t dY[] = {0, 0, 1, -1};

void build_compression_tables() {
	uint32_t stateCnt = 1U << (2 * MAX_L);

	uint32_t maxCompState = 0;
	for (uint32_t i = 0;i < stateCnt;i++) {
		uint32_t compV = i & 3;
		uint32_t mult = 4;
		uint32_t prev = compV;

		bool invalid = false;
		for (uint32_t j = 1;j < MAX_L;j++) {
			uint32_t cur = (i >> (j * 2)) & 3;
			if (cur == (prev ^ 1)) {
				invalid = true;
				break;
			} else if (cur > (prev ^ 1)) {
				cur--;
			}
			prev = (i >> (j * 2)) & 3;
			compV += cur * mult;
			mult *= 3;
		}
		if (invalid) {
			compState[i] = -1;
		} else {
			compState[i] = compV;
			if (compV > maxCompState) {
				maxCompState = compV;
			}
			decompState[compV] = i;
		}
	}
	//std::cerr << maxCompState << "\n";
}

uint32_t food[MAX_N][MAX_N];
uint32_t rest[MAX_N][MAX_N];
uint32_t w, h;
uint32_t k;

uint32_t calc_rest(uint32_t a) {
	uint32_t ans = 1;
	uint32_t c = 1;
	while (c < a) {
		c *= 2;
		ans ++;
	}
	return ans;
}

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> h >> w >> k;
	for (uint32_t y = 0;y < h;y++) {
		for (uint32_t x = 0;x < w;x++) {
			std::cin >> food[x][y];
			rest[x][y] = calc_rest(food[x][y]);

			//std::cerr << rest[x][y] << " ";
		}
		//std::cerr << "\n";
	}
}

inline uint32_t append_state(uint32_t state, uint32_t dir) {
	return ((state << 2) & MOD) + dir;
}

inline bool ok(uint32_t x, uint32_t y, uint32_t nX, uint32_t nY, uint32_t state) {
	for (uint32_t i = 1;i <= MAX_L;i++) {
		x += dX[(state & 3) ^ 1];
		y += dY[(state & 3) ^ 1];
		if ((x == nX) && (y == nY)) {
			return i >= rest[nX][nY];
		}
		state >>= 2;
	}
	return true;
}

int32_t dp[2][MAX_N][MAX_N][COMP_STATE_CNT];

void update_dp(uint32_t l) {
	std::memset(dp[l], -1, sizeof(int32_t) * MAX_N * MAX_N * COMP_STATE_CNT);

	for (uint32_t y = 0;y < h;y++) {
		for (uint32_t x = 0;x < w;x++) {
			for (uint32_t s = 0;s < COMP_STATE_CNT;s++) {
				if (dp[l ^ 1][x][y][s] < 0) {
					continue;
				}
				uint32_t dS = decompState[s];
				for (uint32_t d = 0;d < 4;d++) {
					uint32_t nX = x + dX[d];
					uint32_t nY = y + dY[d];
					if ((nX >= w) || (nY >= h)) {
						continue;
					}
					if (ok(x, y, nX, nY, dS)) {
						uint32_t nS = compState[append_state(dS, d)];
						int32_t curV = dp[l ^ 1][x][y][s] + food[nX][nY];
						if (curV > dp[l][nX][nY][nS]) {
							dp[l][nX][nY][nS] = curV;
						}
					}
				}
			}
		}
	}
}

void init_dp() {
	std::memset(dp[0], -1, sizeof(int32_t) * MAX_N * MAX_N * COMP_STATE_CNT);
	dp[0][0][0][0] = food[0][0];
}
int32_t find_ans(uint32_t l) {
	int32_t ans = 0;
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			for (uint32_t s = 0;s < COMP_STATE_CNT;s++) {
				ans = std::max(ans, dp[l][x][y][s]);
			}
		}
	}
	return ans;
}

int main() {
	build_compression_tables();
	input();
	init_dp();
	//std::cout << find_ans(0) << "\n";
	for (uint32_t i = 1;i < k;i++) {
		update_dp(i & 1);
		//std::cout << find_ans(i & 1) << "\n";
	}
	std::cout << find_ans((k - 1) & 1) << "\n";

}
