/**
* user:  A1001
* fname: Aleksandar
* lname: Krastev
* task:  superstition
* score: 20.0
* date:  2017-11-24 11:19:04.104559
*/
#include <iostream>
#include <cstdint>
#include <vector>
#include <cassert>

const uint64_t MOD = 1e9 + 7;

template <class T, size_t N>
struct Vector {
	T v[N];

	Vector() {}
	Vector(const T& val) {
		for (uint32_t i = 0;i < N;i++) {
			v[i] = val;
		}
	}
	Vector(const Vector<T, N> &a) {
		for (uint32_t i = 0;i < N;i++) {
			v[i] = a.v[i];
		}
	}

	Vector<T, N>& operator%= (uint64_t mod) {
		for (uint32_t i = 0;i < N;i++) {
			v[i] %= mod;
		}
		return *this;
	}

	Vector<T, N>& operator+= (const Vector<T, N> &u) {
		for (uint32_t i = 0;i < N;i++) {
			v[i] += u.v[i];
		}
		return *this;
	}

	friend Vector<T, N> operator+ (Vector<T, N> v, const Vector<T, N> &u) {
		return v += u;
	}
	

	friend T operator*(const Vector<T, N> &v1, const Vector<T, N> &v2) {
		T ans = T();
		for (uint32_t i = 0;i < v1.size();i++) {
			ans += v1.v[i] * v2.v[i];
		}
		return ans;
	}
};

template <class T, size_t N>
struct Matrix {
	T m[N][N];

	Matrix(){}
	Matrix(const T &val) {
		for (uint32_t i = 0;i < N;i++) {
			for (uint32_t j = 0;j < N;j++) {
				m[i][j] = val;
			}
		}
	}

	template<class U>
	Vector<U, N> operator*(const Vector<U, N> &v) {
		Vector<U, N> ans;
		for (uint32_t i = 0;i < N;i++) {
			ans.v[i] = m[i][0] * v.v[0];
			for (uint32_t j = 1;j < N;j++) {
				ans.v[i] += m[i][j] * v.v[j];
				ans.v[i] %= MOD;
			}
		}
		return ans;
	}
	Matrix<T, N>& operator %= (uint64_t num) {
		for (uint32_t i = 0;i < N;i++) {
			for (uint32_t j = 0;j < N;j++) {
				m[i][j] %= num;
			}
		}
		return *this;
	}
	Matrix<T, N>& operator += (const Matrix<T, N> &m1) {
		for (uint32_t i = 0;i < N;i++) {
			for (uint32_t j = 0;j < N;j++) {
				m[i][j] += m1.m[i][j];
			}
		}
		return *this;
	}

	friend Matrix<T, N> operator* (const Matrix<T, N> &m1, const Matrix<T, N> &m2) {
		Matrix<T, N> ans;
		for (uint32_t i = 0;i < N;i++) {
			for (uint32_t j = 0;j < N;j++) {
				ans.m[i][j] = m1.m[i][0] * m2.m[0][j];
				for (uint32_t k = 1;k < N;k++) {
					ans.m[i][j] += m1.m[i][k] * m2.m[k][j];
					ans.m[i][j] %= MOD;
				}
			}
		}
		return ans;
	}
};

//Matrix<Vector<uint64_t>> m;

const uint32_t MAX_D = 1e4 + 10;
const uint32_t MAX_W = 10;
//const uint32_t MAX_W = 2;
const uint32_t MAX_N = 30;
const uint32_t MAX_N2 = 80;


Matrix<Matrix<uint64_t, MAX_N>, MAX_W> distMatrix;
Matrix<Matrix<uint64_t, MAX_N2>, MAX_W> distMatrix2;

uint32_t vCnt;
uint32_t d, k;

void read_matrix(uint32_t eCnt) {
	distMatrix = Matrix<Matrix<uint64_t, MAX_N>, MAX_W>(Matrix<uint64_t, MAX_N>(0));
	for (uint32_t i = 1;i < MAX_W;i++) {
		for (uint32_t j = 0;j < vCnt;j++) {
			distMatrix.m[i][i - 1].m[j][j] = 1;
		}
	}

	for (uint32_t i = 0;i < eCnt;i++) {
		uint32_t a, b, w;
		std::cin >> a >> b >> w;
		a--;
		b--;
		w--;
		distMatrix.m[0][w].m[a][b]++;
		distMatrix.m[0][w].m[b][a]++;
	}
}

void read_matrix2(uint32_t eCnt) {
	distMatrix2 = Matrix<Matrix<uint64_t, MAX_N2>, MAX_W>(Matrix<uint64_t, MAX_N2>(0));
	for (uint32_t i = 1;i < MAX_W;i++) {
		for (uint32_t j = 0;j < vCnt;j++) {
			distMatrix2.m[i][i - 1].m[j][j] = 1;
		}
	}

	for (uint32_t i = 0;i < eCnt;i++) {
		uint32_t a, b, w;
		std::cin >> a >> b >> w;
		a--;
		b--;
		w--;
		distMatrix2.m[0][w].m[a][b]++;
		distMatrix2.m[0][w].m[b][a]++;
	}
}

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	uint32_t eCnt;
	std::cin >> vCnt >> eCnt >> d >> k;
	if (d < MAX_D) {
		read_matrix2(eCnt);
	} else {
		read_matrix(eCnt);
	}
}

Vector<Vector<uint64_t, MAX_N2>, MAX_W> dp[MAX_D];
uint64_t solve_norm_dp() {
	dp[0] = Vector<Vector<uint64_t, MAX_N2>, MAX_W>(Vector<uint64_t, MAX_N2>(0));
	dp[0].v[0] = Vector<uint64_t, MAX_N2>(1);
	uint64_t ans = 0;
	for (uint32_t i = 1;i <= k;i++) {
		dp[i] = distMatrix2 * dp[i - 1];
		//std::cout << dp[i].v.size() << " " << dp[i].v[0].v.size() << std::endl;
		if (i % d == 0) {
			for (uint32_t j = 0;j < vCnt;j++) {
				ans += dp[i].v[0].v[j];
				ans %= MOD;
			}
			//std::cerr << ans << std::endl;
		}
	}
	return ans;
}

template <class T>
T fast_pow(const T& a, uint32_t pow) {
	if (pow == 1) {
		return a;
	}
	T ans = fast_pow(a, pow / 2);
	ans = ans * ans;
	if (pow % 2) {
		ans = ans * a;
	}
	return ans;
}

uint64_t solve_mono_pow() {
	auto curMatrix = fast_pow(distMatrix, d);
	auto base = Vector<Vector<uint64_t, MAX_N>, MAX_W>(Vector<uint64_t, MAX_N>(0));
	base.v[0] = Vector<uint64_t, MAX_N>(1);
	auto ans = curMatrix * base;
	uint64_t ansNum = 0;
	for (uint32_t i = 0;i < vCnt;i++) {
		ansNum += ans.v[0].v[i];
		ansNum %= MOD;
	}
	return ansNum;
}

int main() {
	input();
	if (k < MAX_D) {
		std::cout << solve_norm_dp() << "\n";
	} else {
		std::cout << solve_mono_pow() << "\n";
	}
}
